/********************************************************************************************
 * FEAR: Flexible Embodied Animat 'Rchitecture
 *
 * Copyright (c) 2002-2004 Cerebral Creations, All Rights Reserved.
 *
 * FEAR is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL), as published by the Free Software Foundation. These files
 * are distributed in the hope that they will be useful, but WITHOUT ANY WARRANTY, without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * attached LICENSE file for details, or read http://www.gnu.org/copyleft/gpl.html.
 *
 * 
 * File:            demos\book\jwha3\Module.cpp
 * Description:
 *     Jwha3 is an animat that uses the motion interface to request constant turns and
 *     moves. The motion interface is the only effector available. The results is a behavior
 *     that spins around in circles.
 * Author:          Alex J. Champandard 
 * Contact:         alex@ai-depot.com   
 * Revisions:       See bottom of file.
 *
 ********************************************************************************************/

#include <fear/core.hpp>
#include <fear/services/execute.hpp>
#include "Brain.h"


namespace fear {


// ==========================================================================================
// Jwha3BrainManager
// ------------------------------------------------------------------------------------------
/// Wrapper class that implements all of the glue-code to get the module working.  
// ==========================================================================================
class Jwha3BrainManager : public Jwha3Brain
{

// ------------------------------------------------------------------------------------------
// Persistence & Interface Management
// ------------------------------------------------------------------------------------------

public:

	/**
	 * Return a new interface of a given type.
	 */
	Interface* Export(const std::string& name);

	/**
	 * Store dependencies for convenience, as passed by the architecture.
	 */
	bool Import(const std::string& name, Interface* i);


// ------------------------------------------------------------------------------------------
// Life-Cycle Management
// ------------------------------------------------------------------------------------------

	Jwha3BrainManager()
	:	Jwha3Brain() 
	{
	} // Jwha3BrainManager()

	~Jwha3BrainManager()
	{
	} // ~Jwha3BrainManager()


}; // class Jwha3BrainManager


// ==========================================================================================
// Jwha3BrainExecute
// ------------------------------------------------------------------------------------------
/// Single entry point of control. This interface can be used by most components
/// as a standard hook, including Brain modules.
// ==========================================================================================
class Jwha3BrainExecute : public Interface, public services::iExecute
{

// ------------------------------------------------------------------------------------------
// Public Methods
// ------------------------------------------------------------------------------------------

public:

	/**
	 * Perform the most primitive operation and return within finite time.  This generic 
	 * interface simply calls the brain's Think() function for convenience.
	 */
	void Tick()
	{
		try 
		{
			Architecture& a = dynamic_cast<Architecture&>(component);
			a.Update();
		}
		catch (std::bad_cast) {};
		component.Think();

	} // Tick()


// ------------------------------------------------------------------------------------------
// Life-Cycle Management
// ------------------------------------------------------------------------------------------

	/**
	 * Constructor called by the module when it creates a new interface.
	 */
	Jwha3BrainExecute(Jwha3Brain& c)
	:	Interface( "Execute", 0x0001 ),
		component(c)
	{
	} // Jwha3BrainExecute()

	/**
	 * Empty destructor is virtual to assure consistency of inheritance.
	 */
	virtual ~Jwha3BrainExecute()
	{
	} // ~Jwha3BrainExecute()


// ------------------------------------------------------------------------------------------
// Member Variables
// ------------------------------------------------------------------------------------------

private:

	Jwha3Brain& component; ///< owner of this unique interface

	
}; // class Jwha3BrainExecute


} // namespace fear


using namespace fear;


// ==========================================================================================
// Export
// ------------------------------------------------------------------------------------------
/// Return a new interface of a given type. 
// ==========================================================================================
Interface* Jwha3BrainManager::Export(const std::string& name)
{
    if (name == "Execute")
	{
		Interface* i = dynamic_cast<Interface*>(new Jwha3BrainExecute(*this));
		return i;
	}

	Warning( "Component " + m_type + " does not export interface '" + name + "'.", __FILE__, __LINE__ );
	return NULL;

} // Export()


// ==========================================================================================
// Import
// ------------------------------------------------------------------------------------------
// Find a matching interface, and store the reference for later use. 
// ==========================================================================================
bool Jwha3BrainManager::Import(const std::string& name, Interface* i)
{
    if (name == "motion"  &&  Interface::isCompatible(i->GetVersion(), 0x0004))
	{
        motion = dynamic_cast<body::motion::iRelative*>(i);
        if (motion) return true;
    }
    if (name == "physics"  &&  Interface::isCompatible(i->GetVersion(), 0x0003))
	{
        physics = dynamic_cast<body::physics::iRelative*>(i);
        if (physics) return true;
    }
	if (name == "space"  &&  Interface::isCompatible(i->GetVersion(), 0x0001))
	{
		space = dynamic_cast<body::vision::iSpace*>(i);
		if (space) return true;
	}
    if (name == "visual"  &&  Interface::isCompatible(i->GetVersion(), 0x0004))
	{
        visual = dynamic_cast<body::vision::iSymbolic*>(i);
	    i->RegisterListener( this );
        if (visual) return true;
    }
    if (name == "debug"  &&  Interface::isCompatible(i->GetVersion(), 0x0001))
	{
        debug = dynamic_cast<services::debug::iVisual*>(i);
        if (debug) return true;
    }
  
    Warning( "Component " + m_type + " cannot import interface '" + name + "'.", __FILE__, __LINE__ );
    return false;

} // Import()




// ==========================================================================================
// EXPORT_MODULE
// ------------------------------------------------------------------------------------------
/// This macro automatically registers the Brain with the framework, inside a global factory.
/// It is made available by static linking to the platform, or via a DLL depending on the 
/// linking options.
// ==========================================================================================
EXPORT_MODULE(Jwha3BrainManager);

